# 用户指南

> 原文： [http://docs.cython.org/en/latest/src/userguide/index.html](http://docs.cython.org/en/latest/src/userguide/index.html)

内容：

*   [语言基础知识](language_basics.html)
    *   [声明数据类型](language_basics.html#declaring-data-types)
    *   [C 变量和类型定义](language_basics.html#c-variable-and-type-definitions)
    *   [Python 函数与 C 函数](language_basics.html#python-functions-vs-c-functions)
    *   [自动类型转换](language_basics.html#automatic-type-conversions)
    *   [陈述和表达](language_basics.html#statements-and-expressions)
    *   [Cython 文件类型](language_basics.html#cython-file-types)
    *   [条件编译](language_basics.html#conditional-compilation)
*   [扩展类型](extension_types.html)
    *   [简介](extension_types.html#introduction)
    *   [静态属性](extension_types.html#static-attributes)
    *   [动态属性](extension_types.html#dynamic-attributes)
    *   [类型声明](extension_types.html#type-declarations)
    *   [扩展类型和无](extension_types.html#extension-types-and-none)
    *   [特殊方法](extension_types.html#special-methods)
    *   [属性](extension_types.html#properties)
    *   [子类](extension_types.html#subclassing)
    *   [C 方法](extension_types.html#c-methods)
    *   [前向声明扩展类型](extension_types.html#forward-declaring-extension-types)
    *   [快速实例化](extension_types.html#fast-instantiation)
    *   [来自现有 C / C ++指针的实例化](extension_types.html#instantiation-from-existing-c-c-pointers)
    *   [使扩展类型弱引用](extension_types.html#making-extension-types-weak-referenceable)
    *   [控制 CPython](extension_types.html#controlling-deallocation-and-garbage-collection-in-cpython) 中的释放和垃圾收集
    *   [控制酸洗](extension_types.html#controlling-pickling)
    *   [公共和外部扩展类型](extension_types.html#public-and-external-extension-types)
    *   [公共扩展类型](extension_types.html#public-extension-types)
*   [扩展类型的特殊方法](special_methods.html)
    *   [声明](special_methods.html#declaration)
    *   [Docstrings](special_methods.html#docstrings)
    *   [初始化方法：`__cinit__()`和`__init__()`](special_methods.html#initialisation-methods-cinit-and-init)
    *   [终结方法：`__dealloc__()`](special_methods.html#finalization-method-dealloc)
    *   [算术方法](special_methods.html#arithmetic-methods)
    *   [丰富的比较](special_methods.html#rich-comparisons)
    *   [`__next__()`方法](special_methods.html#the-next-method)
    *   [特殊方法表](special_methods.html#special-method-table)
*   [在 Cython 模块之间共享声明](sharing_declarations.html)
    *   [定义和实施文件](sharing_declarations.html#definition-and-implementation-files)
    *   [定义文件包含什么](sharing_declarations.html#what-a-definition-file-contains)
    *   [实施文件包含什么](sharing_declarations.html#what-an-implementation-file-contains)
    *   [cimport 声明](sharing_declarations.html#the-cimport-statement)
    *   [分享 C 函数](sharing_declarations.html#sharing-c-functions)
    *   [分享扩展类型](sharing_declarations.html#sharing-extension-types)
*   [与外部 C 代码接口](external_C_code.html)
    *   [外部声明](external_C_code.html#external-declarations)
    *   [使用 C](external_C_code.html#using-cython-declarations-from-c) 的 Cython 声明
*   [源文件和编译](source_files_and_compilation.html)
    *   [从命令行编译](source_files_and_compilation.html#compiling-from-the-command-line)
    *   [基本 setup.py](source_files_and_compilation.html#basic-setup-py)
    *   [包中的多个 Cython 文件](source_files_and_compilation.html#multiple-cython-files-in-a-package)
    *   [集成多个模块](source_files_and_compilation.html#integrating-multiple-modules)
    *   [使用`pyximport`](source_files_and_compilation.html#compiling-with-pyximport) 进行编译
    *   [使用`cython.inline`](source_files_and_compilation.html#compiling-with-cython-inline) 进行编译
    *   [与 Sage](source_files_and_compilation.html#compiling-with-sage) 一起编译
    *   [使用 Jupyter 笔记本进行编译](source_files_and_compilation.html#compiling-with-a-jupyter-notebook)
    *   [编译器指令](source_files_and_compilation.html#compiler-directives)
*   [早期绑定速度](early_binding_for_speed.html)
*   [在 Cython 中使用 C ++](wrapping_CPlusPlus.html)
    *   [概述](wrapping_CPlusPlus.html#overview)
    *   [一个简单的教程](wrapping_CPlusPlus.html#a-simple-tutorial)
    *   [编制和导入](wrapping_CPlusPlus.html#compilation-and-importing)
    *   [高级 C ++功能](wrapping_CPlusPlus.html#advanced-c-features)
    *   [RTTI 和 typeid（）](wrapping_CPlusPlus.html#rtti-and-typeid)
    *   [在 setup.py](wrapping_CPlusPlus.html#specify-c-language-in-setup-py) 中指定 C ++语言
    *   [警告和限制](wrapping_CPlusPlus.html#caveats-and-limitations)
*   [融合类型（模板）](fusedtypes.html)
    *   [快速入门](fusedtypes.html#quickstart)
    *   [声明融合类型](fusedtypes.html#declaring-fused-types)
    *   [使用融合类型](fusedtypes.html#using-fused-types)
    *   [选择专业化](fusedtypes.html#selecting-specializations)
    *   [内置熔断类型](fusedtypes.html#built-in-fused-types)
    *   [熔铸函数](fusedtypes.html#casting-fused-functions)
    *   [类型检查专业化](fusedtypes.html#type-checking-specializations)
    *   [条件 GIL 获取/释放](fusedtypes.html#conditional-gil-acquiring-releasing)
    *   [__signatures__](fusedtypes.html#signatures)
*   [将 Cython 代码移植到 PyPy](pypy.html)
    *   [参考计数](pypy.html#reference-counts)
    *   [物件寿命](pypy.html#object-lifetime)
    *   [借用参考和数据指针](pypy.html#borrowed-references-and-data-pointers)
    *   [内置类型，插槽和字段](pypy.html#builtin-types-slots-and-fields)
    *   [GIL 处理](pypy.html#gil-handling)
    *   [效率](pypy.html#efficiency)
    *   [已知问题](pypy.html#known-problems)
    *   [错误和崩溃](pypy.html#bugs-and-crashes)
*   [限制](limitations.html)
    *   [嵌套元组参数解包](limitations.html#nested-tuple-argument-unpacking)
    *   [检查支持](limitations.html#inspect-support)
    *   [堆叠帧](limitations.html#stack-frames)
    *   [推断文字的身份与平等](limitations.html#identity-vs-equality-for-inferred-literals)
*   [Cython 和派热克斯之间的差异](pyrex_differences.html)
    *   [Python 3 支持](pyrex_differences.html#python-3-support)
    *   [条件表达式“x if b else y”](pyrex_differences.html#conditional-expressions-x-if-b-else-y)
    *   [cdef inline](pyrex_differences.html#cdef-inline)
    *   [声明转让（例如“cdef int spam = 5”）](pyrex_differences.html#assignment-on-declaration-e-g-cdef-int-spam-5)
    *   for 循环中的['by'表达式（例如“for i from 0＆lt; = i＆lt; 10 by 2”）](pyrex_differences.html#by-expression-in-for-loop-e-g-for-i-from-0-i-10-by-2)
    *   [Boolean int 类型（例如，它的行为类似于 c int，但是作为布尔值强制转换为/来自 python）](pyrex_differences.html#boolean-int-type-e-g-it-acts-like-a-c-int-but-coerces-to-from-python-as-a-boolean)
    *   [可执行类主体](pyrex_differences.html#executable-class-bodies)
    *   [cpdef 函数](pyrex_differences.html#cpdef-functions)
    *   [自动量程转换](pyrex_differences.html#automatic-range-conversion)
    *   [更友好型铸造](pyrex_differences.html#more-friendly-type-casting)
    *   [cdef / cpdef 函数中的可选参数](pyrex_differences.html#optional-arguments-in-cdef-cpdef-functions)
    *   [结构中的函数指针](pyrex_differences.html#function-pointers-in-structs)
    *   [C ++异常处理](pyrex_differences.html#c-exception-handling)
    *   [同义词](pyrex_differences.html#synonyms)
    *   [源代码编码](pyrex_differences.html#source-code-encoding)
    *   [自动`typecheck`](pyrex_differences.html#automatic-typecheck)
    *   [来自 __future__ 指令](pyrex_differences.html#from-future-directives)
    *   [纯 Python 模式](pyrex_differences.html#pure-python-mode)
*   [类型记忆视图](memoryviews.html)
    *   [快速入门](memoryviews.html#quickstart)
    *   [使用记忆库视图](memoryviews.html#using-memoryviews)
    *   [与旧缓冲支持](memoryviews.html#comparison-to-the-old-buffer-support)的比较
    *   [Python 缓冲支持](memoryviews.html#python-buffer-support)
    *   [内存布局](memoryviews.html#memory-layout)
    *   [Memoryviews 和 GIL](memoryviews.html#memoryviews-and-the-gil)
    *   [Memoryview 对象和 Cython 阵列](memoryviews.html#memoryview-objects-and-cython-arrays)
    *   [Cython 阵列](memoryviews.html#cython-arrays)
    *   [Python 数组模块](memoryviews.html#cpython-array-module)
    *   [强制对 NumPy](memoryviews.html#coercion-to-numpy)
    *   [无切片](memoryviews.html#none-slices)
    *   [通过指针](memoryviews.html#pass-data-from-a-c-function-via-pointer)从 C 函数传递数据
*   [实施缓冲协议](buffer.html)
    *   [矩阵类](buffer.html#a-matrix-class)
    *   [内存安全和引用计数](buffer.html#memory-safety-and-reference-counting)
    *   [旗帜](buffer.html#flags)
    *   [参考文献](buffer.html#references)
*   [使用并行](parallelism.html)
    *   [编制](parallelism.html#compiling)
    *   [突破循环](parallelism.html#breaking-out-of-loops)
    *   [使用 OpenMP 函数](parallelism.html#using-openmp-functions)
*   [调试你的 Cython 程序](debugging.html)
    *   [运行调试器](debugging.html#running-the-debugger)
    *   [使用调试器](debugging.html#using-the-debugger)
    *   [便利功能](debugging.html#convenience-functions)
    *   [配置调试器](debugging.html#configuring-the-debugger)
*   [Cython for NumPy 用户](numpy_tutorial.html)
    *   [Cython 一览](numpy_tutorial.html#cython-at-a-glance)
    *   [您的 Cython 环境](numpy_tutorial.html#your-cython-environment)
    *   [安装](numpy_tutorial.html#installation)
    *   [手动编译](numpy_tutorial.html#manual-compilation)
    *   [第一个 Cython 程序](numpy_tutorial.html#the-first-cython-program)
    *   [添加类型](numpy_tutorial.html#adding-types)
    *   [内存视图的高效索引](numpy_tutorial.html#efficient-indexing-with-memoryviews)
    *   [进一步调整索引](numpy_tutorial.html#tuning-indexing-further)
    *   [将 NumPy 数组声明为连续](numpy_tutorial.html#declaring-the-numpy-arrays-as-contiguous)
    *   [使功能更清晰](numpy_tutorial.html#making-the-function-cleaner)
    *   [更通用的代码](numpy_tutorial.html#more-generic-code)
    *   [使用多线程](numpy_tutorial.html#using-multiple-threads)
    *   [从哪里开始？](numpy_tutorial.html#where-to-go-from-here)
*   [Pythran 作为 Numpy 后端](numpy_pythran.html)
    *   [distutils](numpy_pythran.html#usage-example-with-distutils) 的用法示例

## 指数和表格

*   [指数](../../genindex.html)
*   [模块索引](../../py-modindex.html)
*   [搜索页](../../search.html)